**********************************************************
*
*     PROJECT: XbpBrowseMultiLine ( Ownerdraw mode )
*        FILE: Multi.prg
*  Programmer: J.A. Diego Kerejeta
*     Created: v.1.0  11/10/11
*              v.1.1  17/10/11  Added support for multiline heading and footing
*
**********************************************************

#include 'Appevent.ch'
#include 'Common.ch'
#include 'Font.ch'
#include 'Xbp.ch'

#include 'PresParam.ch'

#pragma Library( "XppUi2.lib" )

#define _Test

**************
PROCEDURE Test
**************
LOCAL name:= GetFilename()
LOCAL oDlg, oFocus, nEvent, mp1, mp2, oXbp, oBrowse, aStruct, i, imax, bError, cDbe

IF name == NIL
   RETURN
ENDIF

bError := ErrorBlock( {|oErr| Break( oErr ) } )
cDBE   := 'DBFDBE'
FOR i:=1 TO 2
   BEGIN SEQUENCE
      DbUseArea( .T., cDBE, name )
   RECOVER
      IF i== 1
         cDBE := 'FOXDBE'
         LOOP
      ENDIF
   ENDSEQUENCE
   EXIT
NEXT
ErrorBlock( bError )

IF ! Used()
   MsgBox ( 'Unable to open file' + CHR(10) + CHR(10)+ Name )
   QUIT
ENDIF

oDlg:= GuiStdDialog()
   oBrowse := GuiBrowseDb( oDlg:drawingArea )

imax   := FCount()
aStruct:= DbStruct()

FOR i:=1 TO imax                           
   IF ! aStruct[i,2] $ 'BOVTXY'
      oBrowse:addColumn( FieldBlockTrimmed( aStruct[ i, 1 ], aStruct[ i, 2 ] ),, { aStruct[ i, 1 ], aStruct[ i, 1 ] }, ;
                         { Lower( aStruct[ i, 1 ] ), Lower( aStruct[ i, 1 ] ) } )
   ENDIF
NEXT
oBrowse:create():show()
oBrowse:itemSelected:= {|u1,u2, obj| GetSelection( oDlg, obj ) }

oDlg:drawingArea:resize := {|mp1,mp2,obj| obj:childList()[1]:setSize(mp2) }
oDlg:setModalState( XBP_DISP_APPMODAL )
oDlg:show()

oFocus:= SetAppFocus( oBrowse )

DO WHILE nEvent <> xbeP_Close
   nEvent := AppEvent( @mp1, @mp2, @oXbp )
   oXbp:handleEvent( nEvent, mp1, mp2 )
ENDDO

oDlg:setModalState( XBP_DISP_MODELESS )
oDlg:destroy()
SetAppFocus( oFocus )
RETURN

***************************************
***************************************
CLASS XbpBrowseMultiLine FROM XbpBrowse
***************************************
***************************************
EXPORTED:
   VAR nInterline
   VAR nLines

   INLINE METHOD destroy
   *********************
      ::XbpBrowse:destroy()
      ::nInterline:= ::nLines:= nil
   RETURN self

   INLINE METHOD getData
   *********************
      LOCAL aData:= {}
      LOCAL nCont

      IF ::cursorMode == XBPBRW_CURSOR_CELL
         Aadd( aData, Eval( ::getColumn( ::colPos ):dataLink ) )
      ELSE
         FOR nCont:= 1 TO ::colCount
            Aadd( aData, Eval( ::getColumn( nCont ):dataLink ) )
         NEXT
      ENDIF
   RETURN aData

   INLINE METHOD AddColumn( obCol, nWidth, xHeading, xFooting, nType )
   *******************************************************************
      LOCAL aPP, oColumn

      IF ValType( obCol ) = 'O'
         IF ! obCol:isDerivedFrom( 'XbpBrowseMultiLine' ) ; obCol:setParent( self ) ; ENDIF
         oColumn:= obCol
      ELSE

#ifndef _Test
                  //
                  //    Works fine
        aPP:= {}
        IF nWidth   <> nil ; Aadd( aPP, { XBP_PP_COL_DA_CHARWIDTH, nWidth } ) ; ENDIF
        IF xHeading <> nil ; Aadd( aPP, { XBP_PP_COL_HA_CAPTION, xHeading } ) ; ENDIF
        IF xFooting <> nil ; Aadd( aPP, { XBP_PP_COL_FA_CAPTION, xFooting } ) ; ENDIF
#else
                  //
                  //    Craches in XbpColumnMultiLine:create() Statement RETURN
        aPP       := cabBROW_PP_COLS
        if nWidth <> NIL
           aPP[COL_DA_CHARWIDTH, 02] := nWidth                  
        endif                                                   
        if xHeading <> NIL                                      
           aPP[COL_HA_CAPTION, 02] := xHeading                  
        endif                                                   
        if xFooting <> NIL                                      
           aPP[COL_FA_CAPTION, 02] := xFooting
        endif
		
#endif


         oColumn:= XbpColumnMultiLine():new( self,,,, aPP )

         IF ValType( xHeading ) = 'A' ; oColumn:heading:= XbpHFMultiLine():new( oColumn ) ; ENDIF
         IF ValType( xFooting ) = 'A' ; oColumn:footing:= XbpHFMultiLine():new( oColumn ) ; ENDIF
         IF nType <> nil              ; oColumn:type   := nType                           ; ENDIF

         oColumn:datalink := obCol
      ENDIF
   RETURN ::XbpBrowse:AddColumn( oColumn )

   INLINE METHOD CustomDrawCell( oPS, aInfo )
   ******************************************
      LOCAL xData, nY, nAlign

      IF aInfo[ XBP_DRAWINFO_AREA ] = aInfo[ XBP_DRAWINFO_COLUMN ]:dataArea
         nAlign:= XBPALIGN_LEFT
      ELSEIF aInfo[ XBP_DRAWINFO_AREA ] = aInfo[ XBP_DRAWINFO_COLUMN ]:heading
         nAlign:= XBPALIGN_HCENTER
      ELSEIF aInfo[ XBP_DRAWINFO_AREA ] = aInfo[ XBP_DRAWINFO_COLUMN ]:footing
         nAlign:= XBPALIGN_RIGHT
      ENDIF

*     IF aInfo[ XBP_DRAWINFO_AREA ] = aInfo[ XBP_DRAWINFO_COLUMN ]:dataArea
         xData:= aInfo[ XBP_DRAWINFO_AREA ]:getCell( aInfo[ XBP_DRAWINFO_ITEM ] )

         IF Valtype( xData ) = 'A'
            nY   := aInfo[ XBP_DRAWINFO_RECT, 4 ] - ::nInterline
            oPS:setColor( IIF( aInfo[ XBP_DRAWINFO_STATE ] = XBP_DRAWSTATE_SELECTED, XBPSYSCLR_MENUHILITE, XBPSYSCLR_WINDOWSTATICTEXT ) )

            AEval( xData, {|xDat| GraCaptionStr( oPS, { aInfo[ XBP_DRAWINFO_RECT, 1 ], nY }, { aInfo[ XBP_DRAWINFO_RECT, 3 ], nY + ::nInterline }, ;
                                                 Var2LChar( xDat ), nAlign ), nY -= ::nInterline } )
            RETURN .F.
         ENDIF
*     ELSEIF aInfo[ XBP_DRAWINFO_AREA ] = aInfo[ XBP_DRAWINFO_COLUMN ]:heading
*     ELSEIF aInfo[ XBP_DRAWINFO_AREA ] = aInfo[ XBP_DRAWINFO_COLUMN ]:footing
*     ENDIF
   RETURN .T.
ENDCLASS

***************************************
CLASS XbpColumnMultiLine FROM XbpColumn
***************************************
EXPORTED:
   INLINE METHOD init( oParent, oOwner, aPos, aSize, aPP, lVisible )
   *****************************************************************
      ::XbpColumn:Init( oParent, oOwner, aPos, aSize, aPP, lVisible )
         ::dataLink  := {|| { '', '' } }
         ::drawMode  := XBP_DRAW_OWNER
   RETURN self

   INLINE METHOD create( oParent, oOwner, aPos, aSize, aPP, lVisible )
   *******************************************************************
      LOCAL _dataLink:= ::datalink
      LOCAL oBrowse  := ::setParent()
      LOCAL aCellRect
      DEFAULT aPP TO {}

      IF oBrowse:nInterline != nil
         IF ValType( Eval( ::dataLink ) ) = 'A' ; ::dataLink:= {|| Eval( _dataLink )[ 1 ] } ; ENDIF
      ELSE
         IF ValType( Eval( ::dataLink ) ) = 'A'
            ::dataLink  := {|| Eval( _dataLink )[ 1 ] }
            oBrowse:nLines:= Len( Eval( _dataLink ) )
         ENDIF
         ::XbpColumn:create( oParent, oOwner, aPos, aSize, aPP, lVisible )

         aCellRect         := ::dataArea:cellRect( 1, .T. )
         oBrowse:nInterline:= ( aCellRect[ 4 ] - aCellRect[ 2 ] ) * 1.1
         ::XbpColumn:destroy()
      ENDIF
	  
	  
#ifndef _Test
      Aadd( aPP, { XBP_PP_COL_DA_ROWHEIGHT, oBrowse:nInterline * oBrowse:nLines } )
#else
      //? aPP[COL_DA_ROWHEIGHT, 2] := oBrowse:nInterline * oBrowse:nLines
        i := AScan( aPP, { | x | x[1] = COL_DA_ROWHEIGHT } )
        If ( i = 0 )
           Aadd( aPP, { XBP_PP_COL_DA_ROWHEIGHT, oBrowse:nInterline * oBrowse:nLines } )
        Else
           aPP[i, 2] := oBrowse:nInterline * oBrowse:nLines
        EndiF 		
#endif
      ::XbpColumn:create( oParent, oOwner, aPos, aSize, aPP )
      ::dataLink := _dataLink
   RETURN Self
ENDCLASS

**************************************
CLASS XbpHFMultiLine FROM XbpCellGroup
**************************************
EXPORTED:
  INLINE METHOD Create( oParent, oOwner, aPos, aSize, aPP, lVisible )
  *******************************************************************
     LOCAL nLines:= Len( ::referenceString )

     ::referenceString:= ::referenceString[ 1 ]

     ::XbpCellgroup:create( oParent, oOwner, aPos, aSize, aPP, lVisible )
        aSize     := ::currentSize()
        aSize[ 2 ]:= Int( ( aSize[ 2 ] * nLines * 0.95 ) )
        ::setSize( aSize )
  RETURN Self
ENDCLASS

*****************************************
PROCEDURE GetSelection( oOwner, oBrowse )
*****************************************
LOCAL oFocus:= SetAppFocus()
LOCAL cTxt  := 'Current selection:' + Chr( 10 )  + Chr( 10 ) 
LOCAL aData := oBrowse:getData()
LOCAL nCols := Len( aData )
LOCAL nCont

FOR nCont:= 1 TO nCols
  IF ValType( aData[ nCont ] ) = 'A'
     AEval( aData[ nCont ], {|xDat| cTxt += Var2LChar( xDat ) + ', ' } )
  ELSE
     cTxt += aData[ nCont ]
  ENDIF
  cTxt += Chr( 10 )
NEXT
ConfirmBox( oOwner, cTxt, 'Selection', XBPMB_OK, XBPMB_INFORMATION+XBPMB_APPMODAL+XBPMB_MOVEABLE )
SetAppFocus( oFocus )
RETURN
 

/////////////////////////
//                     //
//  Generic Functions  //
//                     //
/////////////////////////

PROCEDURE AppSys
****************
RETURN

**************
PROCEDURE Main
**************
LOCAL oDlg, nEvent, mp1, mp2, oXbp

oDlg := XbpDialog():new(,, { 0, 0 }, { 790, 600 },, .F. )
   oDlg:taskList := .T.
   oDlg:border   := XBPDLG_SIZEBORDER
   oDlg:title    := 'Test'
   oDlg:drawingArea:setColorBG( XBPSYSCLR_APPWORKSPACE )
   oDlg:create()
   oDlg:setFrameState( XBPDLG_FRAMESTAT_MAXIMIZED )
   SetAppWindow( oDlg )
   SetAppFocus( oDlg )

   Test()

   DO WHILE nEvent <> xbeP_Close
      nEvent := AppEvent( @mp1, @mp2, @oXbp )
      oXbp:handleEvent( nEvent, mp1, mp2 )
   ENDDO
   QUIT
RETURN

****************
PROCEDURE DbeSys
****************
   IF ! DbeLoad( 'DBFDBE' )
      MsgBox( 'Unable to load DBFDBE' )
   ENDIF
   IF ! DbeLoad( 'FOXDBE' )
      MsgBox( 'Unable to load FOXDBE' )
   ENDIF
   DbeSetDefault( 'DBFDBE' )
RETURN

************************************
FUNCTION GuiBrowseDB( oParent, aPP )
************************************
LOCAL oBrowse
 
oBrowse := XbpBrowseMultiLine():new( oParent,,,, aPP, .F. )
  oBrowse:skipBlock     := {|n| DbSkipper(n) }
  oBrowse:goTopBlock    := {| | DbGoTop()    }
  oBrowse:goBottomBlock := {| | DbGoBottom() }
  oBrowse:phyPosBlock   := {| | Recno()      }
  oBrowse:posBlock      := {| | DbPosition()*10    }
  oBrowse:goPosBlock    := {|n| DbGoPosition(n/10) }
  oBrowse:lastPosBlock  := {| | 1000               }
  oBrowse:firstPosBlock := {| | 0                  }
RETURN oBrowse
 
*******************************
FUNCTION GuiStdDialog( cTitle )
*******************************
LOCAL aSize:= { 750, 550 }
LOCAL aPos := CenterPos( aSize, AppDesktop():currentSize() )
LOCAL oDlg
DEFAULT cTitle TO 'Test'
 
oDlg:= XbpDialog():new( AppDeskTop(), SetAppWindow():drawingArea, aPos, aSize,, .F. )
   oDlg:icon     := 1
   oDlg:taskList := .T.
   oDlg:title    := cTitle
   oDlg:drawingArea:clipChildren := .T.
   oDlg:create()
   oDlg:drawingArea:setFontCompoundName( FONT_DEFPROP_SMALL )

RETURN oDlg

**********************
FUNCTION GetFilename()
**********************
LOCAL oDlg, cFile

oDlg:= XbpFiledialog():new()
   oDlg:Title := 'Please select DBF file'
   oDlg:Create()
   cFile:= oDlg:Open( 'C:\ALASKA\XPPW32\SOURCE\SAMPLES\DATA\*.DBF' )

RETURN cFile

***********************************************
FUNCTION FieldBlockTrimmed( cFieldName, cType )
***********************************************
LOCAL bBlock

IF FieldPos( cFieldName ) <> 0
   IF ! '->' $ cFieldName
      cFieldName := 'FIELD->'+cFieldName
   ENDIF
   IF cType = 'C'
      bBlock := &( '{|x| { Rtrim(' + cFieldName + '), Rtrim(' + cFieldName + ') , Rtrim(' + cFieldName + '), Rtrim(' + cFieldName + ') } } ' )
   ELSE
      bBlock := &( '{|x| { ' + cFieldName + ', ' + cFieldName + ',' + cFieldName + ', ' + cFieldName + ' } } ' )
   ENDIF
ENDIF
RETURN bBlock

*************************************
FUNCTION CenterPos( aSize, aRefSize )
*************************************
RETURN { Int( ( aRefSize[ 1 ] - aSize[ 1 ] ) / 2 ), Int( ( aRefSize[ 2 ] - aSize[ 2 ] ) / 2 ) }




